from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from smart_settings.api import register_settings

register_settings(
    namespace='mailer',
    module='mailer.settings',
    settings=[
        {'name': 'LINK_SUBJECT_TEMPLATE', 'global_name': 'MAILER_LINK_SUBJECT_TEMPLATE', 'default': _('Link for document: {{ document }}'), 'description': _('Template for the document link email form subject line.')},
        {'name': 'LINK_BODY_TEMPLATE', 'global_name': 'MAILER_LINK_BODY_TEMPLATE', 'default': _('To access this document click on the following link: <a href="{{ link }}">{{ link }}</a><br /><br />\n\n--------<br />\nThis email has been sent from Mayan EDMS (http://www.mayan-edms.com)'), 'description': _('Template for the document link email form body line.')},
        {'name': 'DOCUMENT_SUBJECT_TEMPLATE', 'global_name': 'MAILER_DOCUMENT_SUBJECT_TEMPLATE', 'default': _('Document: {{ document }}'), 'description': _('Template for the document email form subject line.')},
        {'name': 'DOCUMENT_BODY_TEMPLATE', 'global_name': 'MAILER_DOCUMENT_BODY_TEMPLATE', 'default': _('Attached to this email is the document: {{ document }}<br /><br />\n\n--------<br />\nThis email has been sent from Mayan EDMS (http://www.mayan-edms.com)'), 'description': _('Template for the document email form body line.')},
    ]
)



from __future__ import unicode_literals

from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse

from acls.models import AccessEntry
from permissions.models import Permission


class ExtraContextMixin(object):
    extra_context = {}

    def get_extra_context(self):
        return self.extra_context

    def get_context_data(self, **kwargs):
        context = super(ExtraContextMixin, self).get_context_data(**kwargs)
        context.update(self.get_extra_context())
        return context


class ObjectListPermissionFilterMixin(object):
    object_permission = None

    def get_queryset(self):
        queryset = super(ObjectListPermissionFilterMixin, self).get_queryset()

        if self.object_permission:
            try:
                # Check to see if the user has the permissions globally
                Permission.objects.check_permissions(self.request.user, [self.object_permission])
            except PermissionDenied:
                # No global permission, filter ther queryset per object + permission
                return AccessEntry.objects.filter_objects_by_access(self.object_permission, self.request.user, queryset)
            else:
                # Has the permission globally, return all results
                return queryset
        else:
            return queryset


class ObjectPermissionCheckMixin(object):
    object_permission = None

    def dispatch(self, request, *args, **kwargs):

        if self.object_permission:
            try:
                Permission.objects.check_permissions(request.user, [self.object_permission])
            except PermissionDenied:
                AccessEntry.objects.check_access(self.object_permission, request.user, self.get_object())

        return super(ObjectPermissionCheckMixin, self).dispatch(request, *args, **kwargs)


class RedirectionMixin(object):
    post_action_redirect = None

    def dispatch(self, request, *args, **kwargs):
        self.next_url = self.request.POST.get('next', self.request.GET.get('next', self.post_action_redirect if self.post_action_redirect else self.request.META.get('HTTP_REFERER', reverse('main:home'))))
        self.previous_url = self.request.POST.get('previous', self.request.GET.get('previous', self.request.META.get('HTTP_REFERER', reverse('main:home'))))

        return super(RedirectionMixin, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(RedirectionMixin, self).get_context_data(**kwargs)
        context.update(
            {
                'next': self.next_url,
                'previous': self.previous_url
            }
        )

        return context


class ViewPermissionCheckMixin(object):
    view_permission = None

    def dispatch(self, request, *args, **kwargs):
        if self.view_permission:
            Permission.objects.check_permissions(self.request.user, [self.view_permission])

        return super(ViewPermissionCheckMixin, self).dispatch(request, *args, **kwargs)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _


def is_superuser(context):
    return context['request'].user.is_staff or context['request'].user.is_superuser


maintenance_menu = {'text': _('Maintenance'), 'view': 'main:maintenance_menu', 'famfam': 'wrench', 'icon': 'main/icons/wrench.png'}
admin_site = {'text': _('Admin site'), 'view': 'admin:index', 'famfam': 'keyboard', 'icon': 'main/icons/keyboard.png', 'condition': is_superuser}



from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext


class ModelAttribute(object):
    __registry = {}

    @classmethod
    def get_for(cls, model, type_names=None):
        result = []

        try:
            for type_name, attributes in cls.__registry[model].iteritems():
                if not type_names or type_name in type_names:
                    result.extend(attributes)

            return result
        except IndexError:
            # We were passed a model instance, try again using the model of the
            # instance

            # If we are already in the model class, exit with an error
            if model.__class__ == models.base.ModelBase:
                raise

            return cls.get_for[type(model)]

    @classmethod
    def get_choices_for(cls, model, type_names=None):
        return [(attribute.name, attribute) for attribute in cls.get_for(model, type_names)]

    @classmethod
    def help_text_for(cls, model, type_names=None):
        result = []
        for count, attribute in enumerate(cls.get_for(model, type_names), 1):
            result.append('{}) {}'.format(count, unicode(attribute.get_display(show_name=True))))

        return ' '.join([ugettext('Available attributes: '), ', '.join(result)])

    def get_display(self, show_name=False):
        if self.description:
            return '{} - {}'.format(self.name if show_name else self.label, self.description)
        else:
            return unicode(self.name if show_name else self.label)

    def __unicode__(self):
        return self.get_display()

    def __init__(self, model, name, label=None, description=None, type_name=None):
        self.model = model
        self.label = label
        self.name = name
        self.description = description

        for field in model._meta.fields:
            if field.name == name:
                self.label = field.verbose_name
                self.description = field.help_text

        self.__registry.setdefault(model, {})

        if isinstance(type_name, list):
            for single_type in type_name:
                self.__registry[model].setdefault(single_type, [])
                self.__registry[model][single_type].append(self)
        else:
            self.__registry[model].setdefault(type_name, [])
            self.__registry[model][type_name].append(self)



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from permissions.models import Permission, PermissionNamespace

sources_setup_namespace = PermissionNamespace('sources_setup', _('Sources setup'))
PERMISSION_SOURCES_SETUP_CREATE = Permission.objects.register(sources_setup_namespace, 'sources_setup_create', _('Create new document sources'))
PERMISSION_SOURCES_SETUP_DELETE = Permission.objects.register(sources_setup_namespace, 'sources_setup_delete', _('Delete document sources'))
PERMISSION_SOURCES_SETUP_EDIT = Permission.objects.register(sources_setup_namespace, 'sources_setup_edit', _('Edit document sources'))
PERMISSION_SOURCES_SETUP_VIEW = Permission.objects.register(sources_setup_namespace, 'sources_setup_view', _('View existing document sources'))



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'SmartLink'
        db.create_table(u'linking_smartlink', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=96)),
            ('dynamic_title', self.gf('django.db.models.fields.CharField')(max_length=96, blank=True)),
            ('enabled', self.gf('django.db.models.fields.BooleanField')(default=True)),
        ))
        db.send_create_signal(u'linking', ['SmartLink'])

        # Adding model 'SmartLinkCondition'
        db.create_table(u'linking_smartlinkcondition', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('smart_link', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['linking.SmartLink'])),
            ('inclusion', self.gf('django.db.models.fields.CharField')(default=u'&', max_length=16)),
            ('foreign_document_data', self.gf('django.db.models.fields.CharField')(max_length=32)),
            ('operator', self.gf('django.db.models.fields.CharField')(max_length=16)),
            ('expression', self.gf('django.db.models.fields.TextField')()),
            ('negated', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('enabled', self.gf('django.db.models.fields.BooleanField')(default=True)),
        ))
        db.send_create_signal(u'linking', ['SmartLinkCondition'])

    def backwards(self, orm):
        # Deleting model 'SmartLink'
        db.delete_table(u'linking_smartlink')

        # Deleting model 'SmartLinkCondition'
        db.delete_table(u'linking_smartlinkcondition')

    models = {
        u'linking.smartlink': {
            'Meta': {'object_name': 'SmartLink'},
            'dynamic_title': ('django.db.models.fields.CharField', [], {'max_length': '96', 'blank': 'True'}),
            'enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '96'})
        },
        u'linking.smartlinkcondition': {
            'Meta': {'object_name': 'SmartLinkCondition'},
            'enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'expression': ('django.db.models.fields.TextField', [], {}),
            'foreign_document_data': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'inclusion': ('django.db.models.fields.CharField', [], {'default': "u'&'", 'max_length': '16'}),
            'negated': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'operator': ('django.db.models.fields.CharField', [], {'max_length': '16'}),
            'smart_link': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['linking.SmartLink']"})
        }
    }

    complete_apps = ['linking']



from __future__ import unicode_literals

from django.core.files.base import File
from django.test import TransactionTestCase

from documents.models import Document, DocumentType
from documents.tests import TEST_SMALL_DOCUMENT_PATH, TEST_DOCUMENT_TYPE


class DocumentOCRTestCase(TransactionTestCase):
    def setUp(self):
        self.document_type = DocumentType.objects.create(name=TEST_DOCUMENT_TYPE, ocr=False)

        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document = Document.objects.new_document(file_object=File(file_object), document_type=self.document_type)[0].document

    def _test_ocr_language_issue_16(self, language, result):
        """
        Reusable OCR test for a specific language
        """

        # Clear the document's first page content and switch language
        self.document.language = language
        self.document.save()
        first_page = self.document.pages.first()
        first_page.content = ''
        first_page.save()

        # Queue document for OCR
        self.document.submit_for_ocr()

        # Make sure content was extracted
        self.assertTrue(result in self.document.pages.first().content)

    def test_ocr_language_backends_end(self):
        self._test_ocr_language_issue_16('eng', 'Mayan EDMS')

    # TODO: Add documents with for the corresponding language
    # def test_ocr_language_backends_deu(self):
    #    self._test_ocr_language_issue_16('deu', 'Mayan EDMS')

    # def test_ocr_language_backends_spa(self):
    #    self._test_ocr_language_issue_16('spa', 'Mayan EDMS')

    # def test_ocr_language_backends_rus(self):
    #    self._test_ocr_language_issue_16('rus', '')

    def tearDown(self):
        self.document.delete()
        self.document_type.delete()



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Removing M2M table for field document on 'Tag'
        db.delete_table(db.shorten_name(u'tags_tag_document'))

        # Adding M2M table for field documents on 'Tag'
        m2m_table_name = db.shorten_name(u'tags_tag_documents')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('tag', models.ForeignKey(orm[u'tags.tag'], null=False)),
            ('document', models.ForeignKey(orm[u'documents.document'], null=False))
        ))
        db.create_unique(m2m_table_name, ['tag_id', 'document_id'])

    def backwards(self, orm):
        # Adding M2M table for field document on 'Tag'
        m2m_table_name = db.shorten_name(u'tags_tag_document')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('tag', models.ForeignKey(orm[u'tags.tag'], null=False)),
            ('document', models.ForeignKey(orm[u'documents.document'], null=False))
        ))
        db.create_unique(m2m_table_name, ['tag_id', 'document_id'])

        # Removing M2M table for field documents on 'Tag'
        db.delete_table(db.shorten_name(u'tags_tag_documents'))

    models = {
        u'documents.document': {
            'Meta': {'ordering': "['-date_added']", 'object_name': 'Document'},
            'date_added': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'document_type': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'documents'", 'null': 'True', 'to': u"orm['documents.DocumentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'max_length': '48', 'blank': 'True'})
        },
        u'documents.documenttype': {
            'Meta': {'ordering': "['name']", 'object_name': 'DocumentType'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '32'})
        },
        u'tags.tag': {
            'Meta': {'object_name': 'Tag'},
            'color': ('django.db.models.fields.CharField', [], {'max_length': '3'}),
            'documents': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['documents.Document']", 'symmetrical': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '128', 'db_index': 'True'})
        },
        u'tags.tagproperties': {
            'Meta': {'object_name': 'TagProperties'},
            'color': ('django.db.models.fields.CharField', [], {'max_length': '3'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'tag': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'properties'", 'to': u"orm['tags.Tag']"})
        }
    }

    complete_apps = ['tags']



from __future__ import unicode_literals

import os

from django.conf import settings

from smart_settings.api import register_settings

register_settings(
    namespace='storage',
    module='storage.settings',
    settings=[
        {'name': 'FILESTORAGE_LOCATION', 'global_name': 'STORAGE_FILESTORAGE_LOCATION', 'default': os.path.join(settings.MEDIA_ROOT, 'document_storage'), 'exists': True},
    ]
)



from __future__ import unicode_literals

import logging

from django.contrib.auth.models import User
from django.core.files import File

from mayan.celery import app

from common.models import SharedUploadedFile

from .models import Document, DocumentType, DocumentVersion

logger = logging.getLogger(__name__)


@app.task(compression='zlib')
def task_get_document_image(document_id, *args, **kwargs):
    document = Document.objects.get(pk=document_id)
    return document.get_image(*args, **kwargs)


@app.task(ignore_result=True)
def task_clear_image_cache():
    # TODO: Error logging / notification
    Document.clear_image_cache()


@app.task(ignore_result=True)
def task_update_page_count(version_id):
    document_version = DocumentVersion.objects.get(pk=version_id)
    document_version.update_page_count()


@app.task(ignore_result=True)
def task_new_document(document_type_id, shared_uploaded_file_id, label, description=None, expand=False, language=None, user_id=None):
    shared_uploaded_file = SharedUploadedFile.objects.get(pk=shared_uploaded_file_id)
    document_type = DocumentType.objects.get(pk=document_type_id)

    if user_id:
        user = User.objects.get(pk=user_id)
    else:
        user = None

    with File(file=shared_uploaded_file.file) as file_object:
        Document.objects.new_document(document_type=document_type, expand=expand, file_object=file_object, label=label, description=description, language=language, user=user)

    shared_uploaded_file.delete()

    # TODO: Report/record how was file uploaded
    #    if result['is_compressed'] is None:
    #        messages.success(request, _('File uploaded successfully.'))

    #    if result['is_compressed'] is True:
    #        messages.success(request, _('File uncompressed successfully and uploaded as individual files.'))

    #    if result['is_compressed'] is False:
    #        messages.warning(request, _('File was not a compressed file, uploaded as it was.'))


@app.task(ignore_result=True)
def task_upload_new_version(document_id, shared_uploaded_file_id, user_id, comment=None):
    shared_file = SharedUploadedFile.objects.get(pk=shared_uploaded_file_id)
    document = Document.objects.get(pk=document_id)

    if user_id:
        user = User.objects.get(pk=user_id)
    else:
        user = None

    with File(file=shared_file.file) as file_object:
        document.new_version(comment=comment, file_object=file_object, user=user)

    shared_file.delete()
